package my.trino.udf;

import io.trino.spi.function.Description;
import io.trino.spi.function.ScalarFunction;
import io.trino.spi.function.SqlType;
import io.airlift.slice.Slice;
import io.airlift.slice.Slices;

@ScalarFunction("lcs")
@Description("Returns the longest common subsequence of two comma-separated strings")
public final class LCSFunction {

    @SqlType("varchar")
    public static Slice lcs(@SqlType("varchar") Slice str1, @SqlType("varchar") Slice str2) {
        String s1 = str1.toStringUtf8();
        String s2 = str2.toStringUtf8();
        
        // 콤마로 구분된 문자열을 배열로 분리
        String[] s1Parts = s1.split(",");
        String[] s2Parts = s2.split(",");
        
        // LCS 계산
        String lcsString = computeLCS(s1Parts, s2Parts);
        return Slices.utf8Slice(lcsString);  // 결과를 Slice 타입으로 반환
    }

    private static String computeLCS(String[] s1Parts, String[] s2Parts) {
        int m = s1Parts.length;
        int n = s2Parts.length;
        int[][] dp = new int[m + 1][n + 1];

        // DP 테이블을 계산하여 LCS 문자열을 구함
        for (int i = 1; i <= m; i++) {
            for (int j = 1; j <= n; j++) {
                if (s1Parts[i - 1].equals(s2Parts[j - 1])) {
                    dp[i][j] = dp[i - 1][j - 1] + 1;
                } else {
                    dp[i][j] = Math.max(dp[i - 1][j], dp[i][j - 1]);
                }
            }
        }

        // LCS 문자열 추적
        StringBuilder lcs = new StringBuilder();
        int i = m, j = n;
        while (i > 0 && j > 0) {
            if (s1Parts[i - 1].equals(s2Parts[j - 1])) {
                lcs.insert(0, s1Parts[i - 1] + ",");  // 앞에 추가 (반전하지 않음)
                i--;
                j--;
            } else if (dp[i - 1][j] >= dp[i][j - 1]) {
                i--;
            } else {
                j--;
            }
        }

        // 마지막에 추가된 콤마를 제거
        String result = lcs.toString();
        return result.isEmpty() ? "" : result.substring(0, result.length() - 1);  // 마지막 콤마 제거
    }
}

